package shodan

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestClient_CountExploits_nilOptions(t *testing.T) {
	_, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	_, err := client.CountExploits(context.TODO(), nil)
	assert.NotNil(t, err)
	assert.EqualValues(t, ErrInvalidQuery, err)
}

func TestClient_CountExploits_emptyQuery(t *testing.T) {
	_, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	_, err := client.CountExploits(context.TODO(), &ExploitSearchOptions{})
	assert.NotNil(t, err)
	assert.EqualValues(t, ErrInvalidQuery, err)
}

func TestClient_CountExploits_noFacets(t *testing.T) {
	mux, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	mux.HandleFunc(exploitCountPath, func(w http.ResponseWriter, r *http.Request) {
		assert.Equal(t, "GET", r.Method)
		w.Write(getStub(t, "exploits/exploits_count_no_facets")) //nolint:errcheck
	})

	expectedExploitsCount := &ExploitSearch{Total: 40, Matches: []*Exploit{}}
	options := &ExploitSearchOptions{Query: "port=22"}
	exploitsCount, err := client.CountExploits(context.TODO(), options)

	assert.Nil(t, err)
	assert.Equal(t, expectedExploitsCount, exploitsCount)
}

func TestClient_CountExploits_facets(t *testing.T) {
	mux, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	mux.HandleFunc(exploitCountPath, func(w http.ResponseWriter, r *http.Request) {
		assert.Equal(t, "GET", r.Method)
		w.Write(getStub(t, "exploits/exploits_count_facets")) //nolint:errcheck
	})

	expectedExploitsCount := &ExploitSearch{
		Total:   40,
		Matches: []*Exploit{},
		Facets: map[string][]*Facet{
			"platform": {
				{Count: 24, Value: "Windows"},
				{Count: 5, Value: "OSX"},
			},
			"author": {
				{Count: 9, Value: "juan vazquez <juan.vazquez@metasploit.com>"},
				{Count: 7, Value: "sinn3r <sinn3r@metasploit.com>"},
			},
		},
	}
	options := &ExploitSearchOptions{
		Query:  "type=exploit",
		Facets: "platform,author",
	}
	exploitsCount, err := client.CountExploits(context.TODO(), options)

	assert.Nil(t, err)
	assert.Equal(t, expectedExploitsCount, exploitsCount)
}

func TestClient_SearchExploits_nilOptions(t *testing.T) {
	_, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	_, err := client.SearchExploits(context.TODO(), nil)
	assert.NotNil(t, err)
	assert.EqualValues(t, ErrInvalidQuery, err)
}

func TestClient_SearchExploits_emptyQuery(t *testing.T) {
	_, tearDownTestServe, client := setUpTestServe()
	defer tearDownTestServe()

	_, err := client.SearchExploits(context.TODO(), &ExploitSearchOptions{})
	assert.NotNil(t, err)
	assert.EqualValues(t, ErrInvalidQuery, err)
}
